<?php
/**
 * DocumentTypeController.php.
 *
 * Created By: jonathan
 * Date: 15/10/2020
 * Time: 12:59
 */

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Requests\EvaluationRequest;
use App\Models\Evaluation;
use App\Models\EvaluationMeasure;
use App\Models\User;
use Illuminate\Support\Facades\Auth;

class EvaluationsController extends Controller
{
    public function all()
    {
        /**
         * @var User $authUser
         */
        $user = Auth::user();

        if (User::ROLE_ADMIN === $user->role) {
            return Evaluation::with('dangerLevels')
                ->with('evaluationMeasures')
                ->with('organization')
                ->get();
        }

        return Evaluation::where('organization_id', $user->organization_id)
                ->with('dangerLevels')
                ->with('evaluationMeasures')
                ->with('organization')
                ->get();
    }

    public function get(int $id)
    {
        $user = Auth::user();

        $evaluation = Evaluation::find($id);

        // if user is admin or if user->organization = $id
        if ((User::ROLE_ADMIN === $user->role) || (($user->role > User::ROLE_ADMIN) && ($user->organization_id === $evaluation->organization_id))) {
            return Evaluation::where('id', $id)
                ->with('dangerLevels')
                ->with('evaluationMeasures')
                ->with('organization')
                ->get();
        }

        abort(403);
    }

    /**
     * Enregistre l'évaluation en tant que terminée (validation sur l'étape en cours).
     *
     * @param $id
     *
     * @return void
     */
    public function save(EvaluationRequest $request, $id = null)
    {
        $user = Auth::user();
        $data = $request->validated();

        if ($user->role <= User::ROLE_MANAGER) {
            if ((User::ROLE_MANAGER === $user->role) && ($data['organization_id'] !== $user->organization_id)) {
                abort(403);
            }
            if (!isset($id)) {
                $evaluation = new Evaluation();
                $evaluation->author = $data['author'];
            } else {
                $evaluation = Evaluation::find($id);
            }

            $evaluation->status = $data['status'];
            $evaluation->current_step = $data['current_step'];
            $evaluation->current_step_status = $data['current_step_status'];
            $evaluation->updated_by = $data['updated_by'];
            $evaluation->organization_id = $data['organization_id'];
            $evaluation->save();

            // Evaluer les dangers
            if (1 === $data['current_step']) {
                $levelsIds = [];
                foreach ($data['danger_levels'] as $dangerlevel) {
                    $levelsIds[] = $dangerlevel['id'];
                }
                $evaluation->dangerLevels()->sync($levelsIds);
            }

            // Evaluer les mesures d'évaluation
            if (2 === $data['current_step']) {
                foreach ($data['evaluation_measures'] as $evaluation_measure) {
                    if (null !== $evaluation_measure['id']) {
                        $evaluation_measure = EvaluationMeasure::find($dangerlevel['id']);
                    } else {
                        $evaluation_measure = new EvaluationMeasure();
                    }

                    $evaluation_measure->measure_id = $evaluation_measure['measure_id'];
                    $evaluation_measure->evaluation_id = $evaluation_measure['evaluation_id'];
                    $evaluation_measure->actual_level = $evaluation_measure['actual_level'];
                    $evaluation_measure->expected_level = $evaluation_measure['expected_level'];
                    $evaluation_measure->manager_id = $evaluation_measure['manager_id'];
                    $evaluation_measure->end_date = $evaluation_measure['end_date'];
                    $evaluation_measure->save();
                }
            }

            return Evaluation::where('id', $evaluation->id)
                ->with('dangerLevels')
                ->with('evaluationMeasures')
                ->with('organization')
                ->first();
        }

        abort(403);
    }

    public function delete(int $id = null)
    {
        $user = Auth::user();

        $evaluation = Evaluation::find($id);

        if ((User::ROLE_ADMIN === $user->role) || ((User::ROLE_MANAGER === $user->role) && ($user->organization_id === $evaluation->organization_id))) {
            $evaluation->delete();

            return abort(204);
        }

        abort(403);
    }
}
